// PlayListWidget.cs
//
//  Copyright (C) 2007-2009 Andoni Morales Alastruey
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
//
//

using System;
using Gtk;
using Gdk;
using LongoMatch.Video.Editor;
using Mono.Unix;
using System.IO;
using LongoMatch.Common;
using LongoMatch.Handlers;
using LongoMatch.Store;
using LongoMatch.Video.Player;
using LongoMatch.Video;
using LongoMatch.Video.Common;
using LongoMatch.Gui;
using LongoMatch.Gui.Dialog;
using LongoMatch.Playlist;
using LongoMatch.Services.JobsManager;



namespace LongoMatch.Gui.Component
{

	[System.ComponentModel.Category("LongoMatch")]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class PlayListWidget : Gtk.Bin
	{
		public event PlayListNodeSelectedHandler PlayListNodeSelected;
		public event ApplyCurrentRateHandler ApplyCurrentRate;
		public event JobHandler NewRenderingJob;

		private PlayerBin player;
		private PlayListPlay plNode;
		private PlayList playList;
		private uint timeout;
		private object lock_node;
		private bool clock_started = false;

		public PlayListWidget()
		{
			this.Build();
			lock_node = new System.Object();
			playlisttreeview1.Reorderable = true;
			playlisttreeview1.RowActivated += OnPlaylisttreeview1RowActivated;
			playlisttreeview1.ApplyCurrentRate += OnApplyRate;
			savebutton.Sensitive = false;

			newbutton.CanFocus = false;
			openbutton.CanFocus = false;
			savebutton.CanFocus = false;
			newvideobutton.CanFocus = false;
		}

		public void SetPlayer(PlayerBin player) {
			this.player = player;
			newvideobutton.Hide();
		}

		public void Load(string filePath) {
			try {
				playList = PlayList.Load(filePath);
				Model = playList.GetModel();
				label1.Visible = false;
				newvideobutton.Show();
				playlisttreeview1.PlayList = playList;
				playlisttreeview1.Sensitive = true;
				savebutton.Sensitive = true;
			} catch (Exception e){
				Log.Exception (e);
				MessagePopup.PopupMessage(this,MessageType.Error,Catalog.GetString("The file you are trying to load is not a playlist or it's not compatible with the current version"));
			}
		}

		public ListStore Model {
			set {
				playlisttreeview1.Model = value;
			}
			get {
				return (ListStore)playlisttreeview1.Model;
			}
		}

		public void Add(PlayListPlay plNode) {
			if(playList!=null) {
				Model.AppendValues(plNode);
				playList.Add(plNode);
			}
		}

		public PlayListPlay Next() {
			if(playList.HasNext()) {
				plNode = playList.Next();
				playlisttreeview1.Selection.SelectPath(new TreePath(playList.GetCurrentIndex().ToString()));
				playlisttreeview1.LoadedPlay = plNode;
				if(PlayListNodeSelected != null && plNode.Valid) {
					PlayListNodeSelected(plNode,playList.HasNext());
					StartClock();
				}
				else
					Next();
				return plNode;
			}
			else {
				return null;
			}
		}

		public void Prev() {
			if ((player.AccurateCurrentTime - plNode.Start.MSeconds) < 500) {
				//Seleccionando el elemento anterior si no han pasado mas 500ms
				if (playList.HasPrev()) {
					plNode = playList.Prev();
					playlisttreeview1.Selection.SelectPath(new TreePath(playList.GetCurrentIndex().ToString()));
					playlisttreeview1.LoadedPlay = plNode;
					if(PlayListNodeSelected != null)
						PlayListNodeSelected(plNode,playList.HasNext());
					StartClock();
				}
			}
			else {
				//Nos situamos al inicio del segmento
				player.SeekTo(plNode.Start.MSeconds,true);
				player.Rate=plNode.Rate;
			}
		}

		public void Stop() {
			StopClock();
		}

		void StartClock()	{
			if(player!=null && !clock_started) {
				timeout = GLib.Timeout.Add(20,CheckStopTime);
				clock_started=true;
			}
		}

		private void StopClock() {
			if(clock_started) {
				GLib.Source.Remove(timeout);
				clock_started = false;
			}
		}

		private bool CheckStopTime() {
			lock(lock_node) {
				if(player != null) {
					if(player.AccurateCurrentTime >= plNode.Stop.MSeconds-200) {
						if(Next() == null)
							StopClock();
					}
				}
				return true;
			}
		}
		private PlayListPlay SelectPlayListNode(TreePath path) {

			plNode = playList.Select(Int32.Parse(path.ToString()));
			if(PlayListNodeSelected != null && plNode.Valid) {
				PlayListNodeSelected(plNode,playList.HasNext());
				StartClock();
			}
			return plNode;
		}

		private FileFilter FileFilter {
			get {
				FileFilter filter = new FileFilter();
				filter.Name = Catalog.GetString("LongoMatch playlist");
				filter.AddPattern("*" + Constants.PLAYLIST_EXT);
				return filter;
			}
		}

		protected virtual void OnPlaylisttreeview1RowActivated(object o, Gtk.RowActivatedArgs args)
		{
			playlisttreeview1.LoadedPlay = SelectPlayListNode(args.Path);
		}


		protected virtual void OnSavebuttonClicked(object sender, System.EventArgs e)
		{
			if(playList != null) {
				playList.Save();
			}
		}

		protected virtual void OnOpenbuttonClicked(object sender, System.EventArgs e)
		{
			FileChooserDialog fChooser = new FileChooserDialog(Catalog.GetString("Open playlist"),
			                (Gtk.Window)this.Toplevel,
			                FileChooserAction.Open,
			                "gtk-cancel",ResponseType.Cancel,
			                "gtk-open",ResponseType.Accept);
			fChooser.SetCurrentFolder(MainClass.PlayListDir());
			fChooser.AddFilter(FileFilter);
			fChooser.DoOverwriteConfirmation = true;
			if(fChooser.Run() == (int)ResponseType.Accept)
				Load(fChooser.Filename);
			fChooser.Destroy();
		}

		protected virtual void OnNewbuttonClicked(object sender, System.EventArgs e)
		{
			FileChooserDialog fChooser = new FileChooserDialog(Catalog.GetString("New playlist"),
			                (Gtk.Window)this.Toplevel,
			                FileChooserAction.Save,
			                "gtk-cancel",ResponseType.Cancel,
			                "gtk-save",ResponseType.Accept);
			fChooser.SetCurrentFolder(MainClass.PlayListDir());
			fChooser.AddFilter(FileFilter);

			if(fChooser.Run() == (int)ResponseType.Accept)
				Load(fChooser.Filename);
			fChooser.Destroy();
		}

		protected virtual void OnPlaylisttreeview1DragEnd(object o, Gtk.DragEndArgs args)
		{
			playList.SetModel((ListStore)playlisttreeview1.Model);
		}

		protected virtual void OnNewvideobuttonClicked(object sender, System.EventArgs e)
		{
			Job job;

			if(playList.Count == 0) {
				MessagePopup.PopupMessage(this,MessageType.Warning,
				                          Catalog.GetString("The playlist is empty!"));
				return;
			}

			job = GuiUtils.ConfigureRenderingJob(playList, this);
			if (job != null && NewRenderingJob != null)
					NewRenderingJob(job);
		}

		protected virtual void OnApplyRate(PlayListPlay plNode) {
			if(ApplyCurrentRate != null)
				ApplyCurrentRate(plNode);
		}
	}
}
